#include "stdio.h"
#include "cc1000_drv.h"
#include "sleep.h"

#define CRC8_POLYNOMINAL         0x85  // Polynomial for CRC8.
#define HW_VER  0x04

/*
-- STATUS_CONTROL_REG (slv_reg1):
    ---------------------------------------------------------------------------------
    | 31 | 30 | 29 | 28 | 27 | 26 | 25 | 24 | 23 | 22 | 21 | 20 | 19 | 18 | 17 | 16 |
    ---------------------------------------------------------------------------------
    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |
 RD:| HW |  HW| HW | HW |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |  0 |
    |VER3|VER2|VER1|VER0|    |    |    |    |    |    |    |    |    |    |    |    |
    ---------------------------------------------------------------------------------
    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |
 WR:|  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |
    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |    |
    ---------------------------------------------------------------------------------
    ---------------------------------------------------------------------------------
    | 15 | 14 | 13 | 12 | 11 | 10 |  9 |  8 |  7 |  6 |  5 |  4 |  3 |  2 |  1 |  0 |
    ---------------------------------------------------------------------------------
    |    |    |    |    |    | rx | rx | tx | tx | tx | rx |    |    |    |rx=0|    |
 RD:|  0 |  0 |  0 |  0 |  0 |fifo|fifo|fifo|fifo|done|done|  0 |  0 |  0 |tx=1|busy|
    |    |    |    |    |    |empt|full|empt|full|    |    |    |    |    |mode|    |
    ---------------------------------------------------------------------------------
    |    |    |    |    |    |    |    |    |    |    |    | rx |strt|strt| rx | tx |
 WR:|  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |  x |fifo|next|next|mode|mode|
    |    |    |    |    |    |    |    |    |    |    |    | rd | tx | rx | set| set|
    ---------------------------------------------------------------------------------
*/


//##############################################################################
void ResetZynq(void) {
	int volatile * const unlock_reg_ptr = (int *) 0XF8000008; //SLCR_UNLOCK
	*unlock_reg_ptr = 0xDF0D;
	int volatile * const reset_reg_ptr = (int *) 0xF8000200; //PSS_RST_CTRL
	*reset_reg_ptr = 0x1;
}
//##############################################################################
void ResetFpgaOnly(void) {
	int volatile * const unlock_reg_ptr = (int *) 0XF8000008; //SLCR_UNLOCK
	*unlock_reg_ptr = 0xDF0D;
	int volatile * const reset_reg_ptr = (int *) 0xF8000240; //FPGA_RST_CTRL
	*reset_reg_ptr = 0x1;
	usleep(1000);
	*reset_reg_ptr = 0x0;
}
//##############################################################################
uint8_t GenerateCRC8(const uint8_t *Buf, uint8_t Size) {
  register uint8_t result = 0x00;
  register uint8_t i;
  register uint8_t j;
  for(i=0; i<Size; i++) {
    result ^= Buf[i];
    for(j=0; j<8; j++) {
      if(result & 0x80) result = (result << 1) ^ CRC8_POLYNOMINAL;
      else              result <<= 1;
    }
  }
  return result;
}
//##############################################################################
void WaitMicroSec (int us) { // tuned for 50MHz clock on BRAM
	usleep(us);
	/*
    //For 50MHz clock, program in BRAM
#ifndef PROGRAM_IN_EXTERNAL_RAM
  while (us > 0)
    {
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop");
    us--;
  }
#else
  us = (us * 200)/3;
  while (us > 0)
    {
    //For 50MHz clock, program in external RAM
    asm("nop");
    us--;
  }
#endif
*/
}
//##############################################################################
void WaitMiliSec(int ms) {
  WaitMicroSec(ms*1000);
}
//##############################################################################
/* When lack of memory, #define version (without checking FULL flag) can be used instead */
int Wr_CC1000_Data(uint32_t Val) {
	int timeout=0;
	while (1) {
		if ((Xil_In32(STATUS_CONTROL_REG_ADDR) & TX_FIFO_FULL_MASK)) {
				WaitMiliSec(1);  // wait some time
				timeout++;
				if (timeout>=CC1000_TIMEOUT_MS) return -1;
		} else {
			Xil_Out32(DATA_REG_ADDR, Val);
			return 0;
		}
  }
}
//##############################################################################
int Rd_CC1000_Data(uint32_t* dataFromCC100) {
	int timeout=0;
	while (1) {
		if ((Xil_In32(STATUS_CONTROL_REG_ADDR) & RX_FIFO_EMPTY_MASK)) {
			WaitMiliSec(1);  // wait some time
			timeout++;
			if (timeout>=CC1000_TIMEOUT_MS) return -1;
		} else {
			// Read data from FIFO:
			Xil_Out32(STATUS_CONTROL_REG_ADDR, (uint32_t)(RXTX_RX_FIFO_READ));
		    *dataFromCC100 = Xil_In32(DATA_REG_ADDR);
			return 0; //success
		}
	}
}
//##############################################################################
void Wr_CC1000_CFG(uint8_t Addr, uint8_t Val) {
	WaitMiliSec(2);  // wait some time
	Wr_CC1000_Config((uint32_t)(CFG_WRITE_MASK | (Addr << 8) | Val));
	WaitMiliSec(2);  // wait some time
	while(Rd_CC1000_Config() & CFG_WRITE_MASK);   // Rb, gdy jest 'busy'.
}
//##############################################################################
int Wr_CC1000_CFG_ack(uint8_t Addr, uint8_t Val) {
	Wr_CC1000_CFG(Addr, Val);
	WaitMiliSec(2);  // wait some time
	uint8_t res = Rd_CC1000_CFG(Addr);
	WaitMiliSec(2);
	if (res != Val) {
		xil_printf("Error in Wr_CC1000_CFG_ack: addr=0x%x, written:0x%x != read=0x%x\n\r", Addr, Val, res);
		return -1;
	}
	return 0;
}
//##############################################################################
uint8_t Rd_CC1000_CFG(uint8_t Addr) {
	Wr_CC1000_Config((uint32_t)(Addr << 8));
	WaitMiliSec(2);  // wait some time
	register uint32_t tmp;
	do {
		tmp = Rd_CC1000_Config();
		WaitMiliSec(2);  // wait some time
	} while(tmp & CFG_WRITE_MASK);   // Rb, gdy jest 'busy'.
	DBG("cfg read addr=0x%02x data=0x%02x\n\r", Addr, tmp);
	return (uint8_t)tmp;
}
//##############################################################################
void CC1000_PrintRegs(void) {
	int DETAILED=1;
	uint32_t  s = Rd_CC1000_Status();

	if (DETAILED) {
		int rff= (s & RX_FIFO_FULL_MASK)  ? 1 : 0;
		int rfe= (s & RX_FIFO_EMPTY_MASK) ? 1 : 0;
		int tff= (s & TX_FIFO_FULL_MASK)  ? 1 : 0;
		int tfe= (s & TX_FIFO_EMPTY_MASK) ? 1 : 0;
		int rdo= (s & RXTX_RX_DONE_MASK)  ? 1 : 0;
		int tdo= (s & RXTX_TX_DONE_MASK)  ? 1 : 0;
		int tm = (s & TX_MODE_MASK)       ? 1 : 0;
		int rtb= (s & RXTX_BUSY_MASK)     ? 1 : 0;
		xil_printf("\n\r");
		xil_printf(" -----------------------------------\n\r");
		xil_printf("|RX FIFO|TX FIFO|  done   |mode|BUSY|\n\r");
		xil_printf("|F=%d,E=%d|F=%d,E=%d|RX=%d TX=%d|TX=%d| %d  |\n\r", rff,rfe, tff, tfe, rdo, tdo, tm, rtb);
		xil_printf(" -----------------------------------\n\r");
	}
	xil_printf("STATUS_CONTROL_REG: 0x%08x\n\r", s);
	xil_printf("CONFIG_REG:         0x%08x\n\r", Rd_CC1000_Config());
	xil_printf("ADC_REG:            0x%08x\n\r", Rd_ADC());
 }
//##############################################################################
int Init_CC1000() {
	unsigned int hw_ver = (Rd_CC1000_Status() & HW_VER_MASK) >> HW_VER_SHIFT;
	if (hw_ver != HW_VER) {
		xil_printf("ERROR: incompatible hardware version of CC1000: is 0x%02x, expecting: 0x%02x\n\r", hw_ver, HW_VER);
	}
	// Switch VHDL module to RX mode (line DIO -> Hi-Z):
	DBG("Init_CC1000 start\n\r", NULL);
	Wr_CC1000_Status(RXTX_RX_MODE_ON);
	// Reset TX fifo:
	Wr_CC1000_Status(RXTX_TX_MODE_ON);
	WaitMiliSec(2);
	Wr_CC1000_Status(RXTX_RX_MODE_ON);
	// Write the registers:
	int res;
	res = Wr_CC1000_CFG_ack(CFG_MAIN,      0x3A);   WaitMiliSec(2); if (res) {xil_printf("ERROR Init_CC1000: CFG_MAIN1\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_MAIN,      0x3B);   WaitMiliSec(2); if (res) {xil_printf("ERROR Init_CC1000: \n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ2A,    0x41); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ2A\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ1A,    0xF8); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ1A\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ0A,    0x00); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ0A\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ2B,    0x41); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ2B\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ1B,    0xFC); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ1B\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FREQ0B,    0x9C); if (res) {xil_printf("ERROR Init_CC1000: CFG_FREQ0B\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FSEP1,     0x02); if (res) {xil_printf("ERROR Init_CC1000: CFG_FSEP1\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FSEP0,     0x80); if (res) {xil_printf("ERROR Init_CC1000: CFG_FSEP0\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_CURRENT,   0x44); if (res) {xil_printf("ERROR Init_CC1000: CFG_CURRENT\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FRONT_END, 0x02); if (res) {xil_printf("ERROR Init_CC1000: CFG_FRONT_END\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_PA_POW,    0x0F); if (res) {xil_printf("ERROR Init_CC1000: CFG_PA_POW\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_PLL,       0x48); //if (res) {xil_printf("ERROR Init_CC1000: CFG_PLL\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_LOCK,      0x10); if (res) {xil_printf("ERROR Init_CC1000: CFG_LOCK\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_CAL,       0x26); if (res) {xil_printf("ERROR Init_CC1000: CFG_CAL\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_MODEM2,    0x90); if (res) {xil_printf("ERROR Init_CC1000: CFG_MODEM2\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_MODEM1,    0x6F); if (res) {xil_printf("ERROR Init_CC1000: CFG_MODEM1\n\r"); return -1;}
//	Wr_CC1000_CFG(CFG_MODEM0,    0x54);   // for speed 76.8kBaud
	res = Wr_CC1000_CFG_ack(CFG_MODEM0,    0x55); if (res) {xil_printf("ERROR Init_CC1000: CFG_MODEM0\n\r"); return -1;}  // for speed 38.4kBaud
	res = Wr_CC1000_CFG_ack(CFG_MATCH,     0x70); if (res) {xil_printf("ERROR Init_CC1000: CFG_MATCH\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_FSCTRL,    0x01); if (res) {xil_printf("ERROR Init_CC1000: CFG_FSCTRL\n\r"); return -1;}
	res = Wr_CC1000_CFG_ack(CFG_PRESCALER, 0x00); if (res) {xil_printf("ERROR Init_CC1000: CFG_PRESCALER\n\r"); return -1;}
//	Wr_CC1000_CFG(CFG_TEST6,     0x10);
//	Wr_CC1000_CFG(CFG_TEST5,     0x08);
	res = Wr_CC1000_CFG_ack(CFG_TEST4,     0x3F); if (res) {xil_printf("ERROR Init_CC1000: CFG_TEST4\n\r"); return -1;}
//	Wr_CC1000_CFG(CFG_TEST3,     0x04);
//	Wr_CC1000_CFG(CFG_TEST2,     0x00);
//	Wr_CC1000_CFG(CFG_TEST1,     0x00);
//	Wr_CC1000_CFG(CFG_TEST0,     0x00);
	Calibrate_RX();
	Calibrate_TX();
	TurnOn_RX();
	DBG("Init_CC1000 end\n\r",NULL);
	return 0; //OK
}
//##############################################################################
/**
 * Starts the calibration of the receiver.
 *
 * @return  Status of the calibration (0-failure, 1-success).
 */
uint8_t Calibrate_RX() {
	Wr_CC1000_CFG(CFG_MAIN,    0x11);
	Wr_CC1000_CFG(CFG_CURRENT, 0x44);
	Wr_CC1000_CFG(CFG_PLL,     0x48);
	Wr_CC1000_CFG(CFG_CAL,     0xA6);
	while( !(Rd_CC1000_CFG(CFG_CAL) & 0x08) );  //Do while end-of-calibration bit is not set.
	Wr_CC1000_CFG(CFG_CAL,     0x26);
	return (uint8_t)(Rd_CC1000_CFG(CFG_LOCK) & 0x01);
}
//##############################################################################
/**
 * Starts the calibration of the transmitter.
 *
 * @return  Status of the calibration (0-failure, 1-success).
 */
uint8_t Calibrate_TX() {
	Wr_CC1000_CFG(CFG_MAIN,    0xE1);
	WaitMiliSec(10);  // wait some time
	Wr_CC1000_CFG(CFG_CURRENT, 0x81);
	WaitMiliSec(10);  // wait some time
	Wr_CC1000_CFG(CFG_PLL,     0x48);
	WaitMiliSec(10);  // wait some time
	Wr_CC1000_CFG(CFG_PA_POW,  0x00);
	WaitMiliSec(10);  // wait some time
	Wr_CC1000_CFG(CFG_CAL,     0xA6);
    WaitMiliSec(10);  // wait some time

	while( !(Rd_CC1000_CFG(CFG_CAL) & 0x08) );   // Rb, gdy nie jest ustawiony bit zakonczenia kalibr.
	Wr_CC1000_CFG(CFG_CAL,     0x26);
	return (uint8_t)(Rd_CC1000_CFG(CFG_LOCK) & 0x01);
}
//##############################################################################
/**
 * Switches CC1000 i and VHDL module 'CC1000_RXTX' in RX mode.
 */
void TurnOn_RX() {
	// Switching VHDL module to RX mode (line DIO -> Hi-Z).
	Wr_CC1000_Status(RXTX_RX_MODE_ON);
	Wr_CC1000_Status(RXTX_RESTART_RX);
	Wr_CC1000_CFG(CFG_MAIN,     0x11);
	Wr_CC1000_CFG(CFG_CURRENT,  0x44);
	Wr_CC1000_CFG(CFG_PLL,      0x48);   WaitMicroSec(250);
	Wr_CC1000_Status(RXTX_RESTART_RX);
}
//##############################################################################
/**
 * Switches CC1000 and VHDL module 'CC1000_RXTX' to TX mode.
 */
void TurnOn_TX() {
	Wr_CC1000_CFG(CFG_PA_POW,   0x00);
	Wr_CC1000_CFG(CFG_MAIN,     0xE1);
	Wr_CC1000_CFG(CFG_CURRENT,  0x81);
	Wr_CC1000_CFG(CFG_PLL,      0x48);   WaitMicroSec(250);
	Wr_CC1000_CFG(CFG_PA_POW,   0x0F);   WaitMicroSec(20);

	// Switches VHDL module to TX mode.
	Wr_CC1000_Status(RXTX_TX_MODE_ON);
}
//##############################################################################
/**
 * Switches CC1000 to Power Down mode and VHDL module 'CC1000_RXTX' to rx mode.
 * (line DIO enters the state Hi-Z).
 */
void TurnOn_PD() {
	// Switching VHDL module to RX mode (line DIO -> Hi-Z).
	Wr_CC1000_Status(RXTX_RX_MODE_ON);
	Wr_CC1000_CFG(CFG_MAIN,     0x3F);
	Wr_CC1000_CFG(CFG_PA_POW,   0x00);
}
//##############################################################################
/**
 * Wakes up from Low Power mode. Switches VHDL module 'CC1000_RXTX' to RX mode.
 */
void TurnOff_PD() {
	// Switching VHDL module to RX mode (line DIO -> Hi-Z).
	Wr_CC1000_Status(RXTX_RX_MODE_ON);
	Wr_CC1000_CFG(CFG_MAIN,     0x3B);   WaitMicroSec(2000);
	Wr_CC1000_CFG(CFG_MAIN,     0x39);   WaitMicroSec(200);
}
//##############################################################################
/**
 * Performing the ADC read operation:
 * 1. send request,
 * 2. wait for the end of the processing,
 * 3. return the value.
 *
 * @return  Result of ADC conversion: the full range of 11-0.
 */
uint32_t MakeAdcConv() {
	St_ADC();
	return WaitUntilAdcDone();
}
//##############################################################################
/**
 * Wait for the end of conversion, which has been stared earlier, then return data from read buffer.
 * If read request was not sent before, the previous converion result will be returned.
 * @return  Result of ADC conversion: the full range of 11-0.
 */
uint32_t WaitUntilAdcDone() {
  register uint32_t tmp;
  do {
	  tmp = Rd_ADC();
  } while(tmp & ADC_BUSY_MASK);
  return (uint32_t)((tmp & ADC_DATA_MASK) >> ADC_DATA_SHIFT); // Voltage on the input of ADC is in range 0-1,2V, - uses 11 LSB bits out of total of 12 bits.
}
//##############################################################################
