#include "xparameters.h"
#include "xil_types.h"
#include "xil_io.h"


#define CC1000_BASEADDR   XPAR_CC1000_0_S00_AXI_BASEADDR
#define CC1000_TIMEOUT_MS 1000

#define PROGRAM_IN_EXTERNAL_RAM // delay is slower when running program from external RAM
//##############################################################################
// PLEASE DO NOT CHANGE BELOW THIS LINE
//##############################################################################
//#define DEBUG_MESSAGES
#ifdef DEBUG_MESSAGES
//#define INFO(fmt, args...) xil_printf("%s:%d "fmt,__FILE__,__LINE__,args)
#define DBG(fmt, args...) xil_printf("DBG(%s:%d): "fmt,__PRETTY_FUNCTION__,__LINE__,args)
#else
#define DBG(fmt, args...)
#endif

#define DATA_REG_ADDR                   (CC1000_BASEADDR)
#define STATUS_CONTROL_REG_ADDR         (CC1000_BASEADDR + 0x00000004)
#define CONFIG_REG_ADDR                 (CC1000_BASEADDR + 0x00000008)
#define ADC_REG_ADDR                    (CC1000_BASEADDR + 0x0000000c)

#define HW_VER_MASK               0xF0000000
#define HW_VER_SHIFT              28
#define RXTX_RX_MODE_ON           0x00000002
#define RXTX_TX_MODE_ON           0x00000001
#define RXTX_RESTART_RX           0x00000004
#define RXTX_RESTART_TX           0x00000008
#define RXTX_RX_FIFO_READ         0x00200010    // prepare next data for reading from FIFO
#define RXTX_BUSY_MASK            0x00000001
#define TX_MODE_MASK              0x00000002
#define RXTX_RX_DONE_MASK         0x00000020
#define RXTX_TX_DONE_MASK         0x00000040
#define TX_FIFO_FULL_MASK         0x00000080
#define TX_FIFO_EMPTY_MASK        0x00000100
#define RX_FIFO_FULL_MASK         0x00000200
#define RX_FIFO_EMPTY_MASK        0x00000400

#define RXTX_BUSY_BIT      0
#define RXTX_MODE_BIT      1
#define RXTX_RX_DONE_BIT   5
#define RXTX_TX_DONE_BIT   6

#define CFG_WRITE_MASK  0x00008000   // Set bir read/write to write to CC1000
#define CFG_MAIN        0x00
#define CFG_FREQ2A      0x01
#define CFG_FREQ1A      0x02
#define CFG_FREQ0A      0x03
#define CFG_FREQ2B      0x04
#define CFG_FREQ1B      0x05
#define CFG_FREQ0B      0x06
#define CFG_FSEP1       0x07
#define CFG_FSEP0       0x08
#define CFG_CURRENT     0x09
#define CFG_FRONT_END   0x0A
#define CFG_PA_POW      0x0B
#define CFG_PLL         0x0C
#define CFG_LOCK        0x0D
#define CFG_CAL         0x0E
#define CFG_MODEM2      0x0F
#define CFG_MODEM1      0x10
#define CFG_MODEM0      0x11
#define CFG_MATCH       0x12
#define CFG_FSCTRL      0x13
#define CFG_PRESCALER   0x1C
#define CFG_TEST6       0x40
#define CFG_TEST5       0x41
#define CFG_TEST4       0x42
#define CFG_TEST3       0x43
#define CFG_TEST2       0x44
#define CFG_TEST1       0x45
#define CFG_TEST0       0x46

#define ADC_BUSY_MASK             0x00008000
#define ADC_NEVER_READ_MASK       0x00004000
#define ADC_DATA_MASK             0x00000FFF
#define ADC_DATA_SHIFT                     0
#define ADC_START_CONVERSION_MASK 0x00000001

#define Wr_CC1000_Status(Val) Xil_Out32(STATUS_CONTROL_REG_ADDR, (uint32_t)(Val))
#define Rd_CC1000_Status() Xil_In32(STATUS_CONTROL_REG_ADDR)
#define Wr_CC1000_Config(Val) Xil_Out32(CONFIG_REG_ADDR, (uint32_t)(Val))
#define Rd_CC1000_Config() Xil_In32(CONFIG_REG_ADDR)
#define St_ADC() Xil_Out32(ADC_REG_ADDR, (uint32_t)ADC_START_CONVERSION_MASK)
#define Rd_ADC() Xil_In32(ADC_REG_ADDR)

//int CC1000Selftest(void);
void ResetZynq(void);
void ResetFpgaOnly(void);
//void ResetCpuOnly(int cpu_no);
uint8_t GenerateCRC8(const uint8_t *Buf, uint8_t Size);
void WaitMicroSec (int us);
void WaitMiliSec(int ms);
int  Wr_CC1000_Data(uint32_t Val);
int Rd_CC1000_Data(uint32_t* dataFromCC100);
void Wr_CC1000_CFG(uint8_t Addr, uint8_t Val);
int Wr_CC1000_CFG_ack(uint8_t Addr, uint8_t Val);
uint8_t Rd_CC1000_CFG(uint8_t Addr);
void CC1000_PrintRegs();
int Init_CC1000();
uint8_t Calibrate_RX();
uint8_t Calibrate_TX();
void TurnOn_RX();
void TurnOn_TX();
void TurnOn_PD();
void TurnOff_PD();

uint32_t MakeAdcConv();
uint32_t WaitUntilAdcDone();


